@@ -18,13 +18,18 @@ Rust MIR: a lowered representation of Rust. Also: an experiment!
 #![crate_type = "rlib"]
 #![crate_type = "dylib"]
 #![deny(warnings)]
-#![unstable(feature = "rustc_private", issue = "27812")]
 
 #![feature(associated_consts)]
 #![feature(box_patterns)]
+#![feature(box_syntax)]
+#![feature(i128_type)]
 #![feature(rustc_diagnostic_macros)]
-#![feature(rustc_private)]
-#![feature(staged_api)]
+#![feature(placement_in_syntax)]
+#![feature(collection_placement)]
+
+#![cfg_attr(stage0, unstable(feature = "rustc_private", issue = "27812"))]
+#![cfg_attr(stage0, feature(rustc_private))]
+#![cfg_attr(stage0, feature(staged_api))]
 
 #[macro_use] extern crate log;
 extern crate graphviz as dot;
@@ -40,15 +45,17 @@ extern crate syntax_pos;
 extern crate rustc_const_math;
 extern crate rustc_const_eval;
 
-extern crate rustc_i128;
-
 pub mod diagnostics;
 
-pub mod build;
-pub mod def_use;
-pub mod graphviz;
+mod build;
 mod hair;
-pub mod mir_map;
-pub mod pretty;
+mod shim;
 pub mod transform;
+pub mod util;
 
+use rustc::ty::maps::Providers;
+
+pub fn provide(providers: &mut Providers) {
+    shim::provide(providers);
+    transform::provide(providers);
+}
